use egui::Color32;
use egui_extras::{Column, TableBuilder};

use crate::manifest::song::SongType;

use super::{context_menu::ContextMenu, ComponentUi};


pub struct SongList;

impl ComponentUi for SongList {
    fn ui(gui: &mut crate::ui::gui::Gui, ui: &mut egui::Ui) {
        let fltr_by;
        let filter_clean;
        if gui.filter.starts_with("playlist:") {
            fltr_by = "playlist";
            filter_clean = gui.filter.strip_prefix("playlist:").unwrap_or("").to_string().to_lowercase();
        } else if gui.filter.starts_with("source:") {
            fltr_by = "source";
            filter_clean = gui.filter.strip_prefix("source:").unwrap_or("").to_string().to_lowercase();
        } else if gui.filter.starts_with("url:") {
            fltr_by = "url";
            filter_clean = gui.filter.strip_prefix("url:").unwrap_or("").to_string();
        } else {
            fltr_by = "";
            filter_clean = gui.filter.clone();
        }

        ui.vertical(|ui| {
            ui.horizontal(|ui| {
                ui.colored_label(Color32::from_hex("#4444aa").unwrap(), "Filter: ");
                ui.text_edit_singleline(&mut gui.filter);
            });  
        });
        
        ui.vertical(|ui| {
            let available_height = ui.available_height();
            let table = TableBuilder::new(ui)
                .striped(true)
                .cell_layout(egui::Layout::left_to_right(egui::Align::Center))
                .resizable(true)
                //.column(Column::auto())
                .column(Column::auto())
                //.column(
                //    Column::remainder()
                //        .at_least(40.0)
                //        .clip(true)
                //        .resizable(true),
                //)
                .column(Column::auto())
                .column(Column::remainder())
                //.column(Column::remainder())
                .min_scrolled_height(0.0)
                .max_scroll_height(available_height)
                .sense(egui::Sense::click());
            
            let playlists = gui.manifest.get_playlists().clone();

            let songs = {
                let mut songs = Vec::new();
                for (pname, p) in playlists {
                    for (sname, s) in p {
                        songs.push((pname.clone(), sname, s));
                    }
                }
                songs
            };

            table.header(20.0, |mut header| {
                // header.col(|_|{});
                header.col(|ui| {
                    ui.strong("Playlist");
                }); 
                header.col(|ui| { 
                    ui.strong("Source");
                });
                header.col(|ui| {
                    ui.strong("Name");
                });
            }).body(|mut body| {
                for (pname, sname, s) in songs {    
                    if fltr_by == "playlist" && !filter_clean.is_empty() {
                        if !pname.to_lowercase().contains(&filter_clean) {
                            continue;
                        }
                    } else if fltr_by == "type" && !filter_clean.is_empty(){
                        if !s.get_type().to_string().to_lowercase().contains(&filter_clean) {
                            continue;
                        }
                    } else if fltr_by == "url" && !filter_clean.is_empty(){
                        if !s.get_url_str().contains(&filter_clean) {
                            continue;
                        }
                    } else if !filter_clean.is_empty() && !sname.to_lowercase().contains(&filter_clean) {
                        continue;
                    }
                    body.row(18.0, |mut row| {
                         
                        row.col(|ui| {
                            ui.label(pname.clone())
                                .context_menu(|ui| ContextMenu::ui(gui, ui, &pname, &sname, &s));
                        });
                        row.col(|ui| {
                            let color =
                            match s.get_type() {
                                SongType::Youtube => Color32::from_hex("#FF0000").unwrap(),
                                SongType::Spotify => Color32::from_hex("#1db954").unwrap(),
                                SongType::Soundcloud => Color32::from_hex("#F26F23").unwrap()
                            };

                            ui.colored_label(color, s.get_type().to_string())
                                .context_menu(|ui| ContextMenu::ui(gui, ui, &pname, &sname, &s));
                        });
                        row.col(|ui| {
                            ui.hyperlink_to(sname.clone(), s.get_url_str())
                                .context_menu(|ui| ContextMenu::ui(gui, ui, &pname, &sname, &s));
                        });

                        row.response()
                            .context_menu(|ui| ContextMenu::ui(gui, ui, &pname, &sname, &s));

                    });
                }
            });
        });

    }
}
