use super::{State, Window};


#[allow(clippy::pedantic)]
#[derive(Debug, Default)]
pub struct GuiImportPlaylist {
    ed_name: String,
    ed_url: String,
}


impl Window for GuiImportPlaylist {
    fn ui(&mut self, state: &mut State, ctx: &egui::Context, open: &mut bool) -> anyhow::Result<()> {
        let mut save = false;
        egui::Window::new("Import Playlist")
            .open(open)
            .show(ctx, |ui| {
                ui.horizontal(|ui| {
                    ui.label("Type: Youtube");
                });

                ui.horizontal(|ui| {
                    ui.label("Name: ");
                    ui.text_edit_singleline(&mut self.ed_name);
                });
                ui.horizontal(|ui| {
                    ui.label("Url: ");
                    ui.text_edit_singleline(&mut self.ed_url);
                });

                if ui.button("Import").clicked() {
                    save = true;
                }
        });

        if save {
            let name = self.ed_name.clone();
            let url = self.ed_url.clone();

            if state.manifest.get_playlist(&name).is_some() {
                log::error!("Playlist {name} already exists");
            }

            let songs = state.downloader.download_playlist_nb(&state.cfg, &url, &name, state.manifest.get_format()).unwrap();
            state.manifest.add_playlist(name.clone());

            let playlist = state.manifest.get_playlist_mut(&name).expect("Unreachable");
            
            for (sname, song) in songs {
                log::info!("Added: {sname}");
                playlist.add_song(sname, song);
            }
            let _ = state.manifest.save(None);
            *open = false;
        }

        Ok(())
    }
}
